<?php

/**
 * Description of cronlog
 *
 * @author hahn
 */

class cronlog {
    
    
    protected $_sDataDir = "__DIR__/data";
    protected $_iTtlCache = 60; // in sec
    
    
    protected $_aServers = false;
    protected $_sActiveServer = false;
    
    protected $_sFileFilter_serverjoblog = '*joblog*.done';
    protected $_sFileFilter_serverlog = '*.log';
    
    protected $_aSkipJoblogs = array(
        // 'dok-kvm-instances',
        'sync-log-2-lithium',
    );
    
    
    public function __construct() {
        if (file_exists(__DIR__.'/cronlog_cfg.php')){
            include(__DIR__.'/cronlog_cfg.php');
        }
        $this->_sDataDir = str_replace("__DIR__", dirname(dirname(__FILE__)), $this->_sDataDir);
        $this->_sDataDir = str_replace('\\', '/', $this->_sDataDir);
        
        $this->getServers();
                
        return true;
    }
    
    // ----------------------------------------------------------------------
    // private
    // ----------------------------------------------------------------------

    
    /**
     * get the full path of directory for caching
     * @return type
     */
    protected function _getCacheDir(){
        return $this->_sDataDir.'/__cache';
    }

    /**
     * get full path of a caching item
     * @param type $sTaskId
     * @return type
     */
    protected function _getCacheFile($sTaskId){
        return $this->_getCacheDir().'/'.$sTaskId;
    }

    /**
     * get full path to a servers cronjob logdata
     * @return type
     */
    protected function _getServerlogDir(){
        return $this->_sDataDir.'/'.$this->_sActiveServer;
    }

    protected function _getCacheData($sTaskId){
        // DISABLE CACHE return false;
        $sFile=$this->_getCacheFile($sTaskId);
        if(file_exists($sFile)){
            if (filemtime($sFile)>(date('U')-$this->_iTtlCache)){
                // echo "USE cache $sFile<br>";
                return unserialize(file_get_contents($sFile));
            } else {
                // echo "DELETE cache $sFile<br>";
                unlink($sFile);
            }
        }
        return false;
    }
    protected function _writeCacheData($sTaskId, $data){
        $sFile=$this->_getCacheFile($sTaskId);
        // echo "WRITE cache $sFile<br>";
        return file_put_contents($sFile, serialize($data));
    }

    // ----------------------------------------------------------------------
    // public getter
    // ----------------------------------------------------------------------
    
    /**
     * get current server
     * @return string
     */
    public function getServer(){
        return $this->_sActiveServer;
    }
    
    /**
     * get array with existing servers in data dir
     * @return array
     */
    public function getServers(){
        
        if(is_array($this->_aServers) && count($this->_aServers)){
            return $this->_aServers;
        }
        
        $this->_aServers=array();
        // echo "DEBUG DATADIR: " . $this->_sDataDir."<br>";
        if (!is_dir($this->_sDataDir)){
            echo "WARNING: no data. variable _sDataDir is wrong (or somehting is not mounted).<br>";
            die();
        }

        if ($handle = opendir($this->_sDataDir)) {
            while (false !== ($entry = readdir($handle))) {
                if ($entry != "." && $entry != ".." && $entry != "__cache" && is_dir($this->_sDataDir.'/'.$entry)) {
                    // echo "DEBUG $entry<br>\n";
                    $this->_aServers[$entry]=array();
                }
            }
            closedir($handle);
        }
        ksort($this->_aServers);
        return $this->_aServers;
    }
    
        /**
         * 
         */
        protected function _parseJoblogLine($sLine){
            $aReturn=array();
            // echo "DEBUG $sLine<br>";
            // job=dok-kvm-instances:host=kalium:start=1538358001:end=1538358001:exectime=0:ttl=60:rc=0
            $sLine=str_replace("\n", '', $sLine);
            $sLine=str_replace(':', '", "', $sLine);
            $sLine=str_replace('=', '": "', $sLine);
            $sLine='{"'.$sLine.'"}';
            // echo "DEBUG $sLine<br><br>";
            $aReturn=json_decode($sLine, 1);
            if(!is_array($aReturn)){
                echo "not a JSON string<br>";
                echo "DEBUG $sLine<br><br>";
                die();
            }
            return $aReturn;
        }
    
    /**
     * get logs from jobfilea of the current or given server
     * @return array
     */
    public function getServerJobHistory(){
        $aReturn=array();
        $sTaskId=__FUNCTION__.'-'.$this->_sActiveServer;

        $aData=$this->_getCacheData($sTaskId);
        if($aData){
            return $aData;
        }

        $aData=array();
            
        foreach(glob($this->_getServerlogDir().'/'.$this->_sFileFilter_serverjoblog) as $sMyJobfile){
            // echo "DEBUG: $sMyJobfile<br>";
            $fileHandle = fopen($sMyJobfile, "r");
            while (($line = fgets($fileHandle)) !== false) {
                // send the current file part to the browser
                $aData=$this->_parseJoblogLine($line);
                if(array_search($aData['job'], $this->_aSkipJoblogs)===false){
                    $aReturn[$aData['start']]=$aData;
                }
            }
            fclose($fileHandle);
        }
        krsort($aReturn);
        $this->_writeCacheData($sTaskId, $aReturn);
        return $aReturn;
    }
    
    protected function _parseLogfile($sFile) {
        $aReturn=array(
            'SCRIPTNAME'=>false,
            'SCRIPTTTL'=>false,
            'SCRIPTSTARTTIME'=>false,
            'SCRIPTLABEL'=>false,
            'SCRIPTENDTIME'=>false,
            'SCRIPTEXECTIME'=>false,
            'SCRIPTRC'=>false,
            // 'SCRIPTOUT'=>array(),
        );
        $fileHandle = fopen($sFile, "r");
        while (($line = fgets($fileHandle)) !== false) {
            // get key ... the part before "="
            $sKey=trim(preg_replace('/=.*/', '', $line));
            if($sKey && isset($aReturn[$sKey])){
                // add value ... the part behind "="
                $aReturn[$sKey]=preg_replace('/^([A-Z]*\=)/', '', $line);
            }
        }
        fclose($fileHandle);
        
        // fetch unit timestamp from date values (they are like "2018-09-30 03:40:05, 1538278805")
        $aReturn['SCRIPTSTARTTIME']=(int)preg_replace('/.*,\ /', '', $aReturn['SCRIPTSTARTTIME']);
        $aReturn['SCRIPTENDTIME']=(int)preg_replace('/.*,\ /', '', $aReturn['SCRIPTSTARTTIME']);
        
        // remove " s" from exec time value
        $aReturn['SCRIPTEXECTIME']=preg_replace('/\ s$/', '', $aReturn['SCRIPTEXECTIME']);
        
        return $aReturn;
        
    }
    
    /**
     * get logs from jobfilea of the current or given server
     * @return array
     */
    public function getServersLastLog(){
        $aReturn=array();
        $aData=array();
        foreach(glob($this->_getServerlogDir().'/'.$this->_sFileFilter_serverlog) as $sMyJobfile){
            // echo "DEBUG: log file $sMyJobfile<br>";
            $aData=$this->_parseLogfile($sMyJobfile);
            $aData['server']=$this->_sActiveServer;
            $aData['logfile']= $this->_sActiveServer.'/'.basename($sMyJobfile);
            $aReturn[$aData['SCRIPTSTARTTIME'].$sMyJobfile]=$aData;
        }
        rsort($aReturn);
        return $aReturn;
    }
    
    // ----------------------------------------------------------------------
    // public setter
    // ----------------------------------------------------------------------
   
    public function setServer($sServer){
        $this->_sActiveServer=false;
        if($sServer==='ALL'){
            return false;
        }
        if($sServer && !array_key_exists($sServer, $this->_aServers)){
            echo "WARNING: server [$sServer] does not exist<br>";
            return false;
        }
        $this->_sActiveServer=$sServer;
        
        return $this->_sActiveServer;
    }
    
    
}